;irsamp.star:  ML core of a learning remote control sampler using run-length
;encoding for samples.  This version uses a 2-nibble repeat count.

;requires a string in stack level 1, and overwrites its contents with the
;sample data.

		header	`x'

		code

		move.a	@d1, a		;put TOS pointer in a
		call	save_regs	;save RPL registers
		call	#01115		;disable interrupts
		call	#01bbd		;turn off display

		move.a	a, d1		;put TOS obj pointer in d1
		move.a	@d1, a		;get obj prolog
		move.p5	type_string, c	;get string prolog
		brne.a	c, a, bailout	;if not a string prolog abort
		add	5, d1		;move past string prolog
		move.a	@d1, c		;get string length field
		add	5, d1		;d1 now points to string data
		sub.a	5, c		;subtract 5 to get length of chars

		move.a	c, b
div3:		srb.a	c		;divide c by 4
		srb.a	c
		add.a	c, b		;add to b
		brnz.a	c, div3		;while c > 0
		srb.a	b		;divide b by 4, to get approx. c/3
		srb.a	b		;c.a is 0
		inc.a	c		;c.a is 1
		brle.a	b, c, bailout	;if b <= 1, no room for samples
		dec.a	b		;trust me, you need this

		move.5	#0011a, d0	;have d0 point to IR LED input nib
		clr.a	c
		move.1	c, @d0		;clear LED status
		move.p5	#80000, c	;move timeout count to c
					;should last about 16 s
wait:		dec.a	c		;c is timeout counter
		brcs	bailout		;if we dec past 0, bail out
checkled:	move.xs	@d0, a		;otherwise, check IR LED input
		brbc	8, a, wait	;if no input, wait for it

sample:		clr.b	c		;c.b is the repeat count
		move.xs	a, c		;copy current input state from a.xs
samploop:	move.xs	@d0, a		;get IR LED status
		inc.x	c		;increment repeat count
		breq.xs	c, a, samploop	;while c.xs is undisturbed, loop
		move.3	c, @d1		;write state and repeat count
		add	3, d1		;increment d1
		dec.a	b		;one less sample
		brcc	sample		;back for new sample
		
bailout:	call	#010e5		;allow interrupts
		call	#01b8f		;turn on display
		jump	rr_rplcont	;back to RPL

		endcode

